<?php
namespace Teknisa\SM\Factory;

use Doctrine\ORM\EntityManager;

class Sm00004000ContratoLocalAtendFactory
{
    
    protected $connection;
    
    public function __construct(EntityManager $entityManager) {
        
        $this->connection = $entityManager->getConnection();
    }
    
    /**
     * A função "validaLocalAtend" verifica se o local de Atendimento 
     * que está sendo relacionado já está relacionado na tabela.
     */
    
    public function validaLocalAtend($params){
        
        $sql = "SELECT COUNT(*) AS VALIDA
                  FROM FCT_RELACESTRUTURACONT
                 WHERE NRESTRUTURAM = :NRESTRUTURAM
                   AND NRSEQCONTRATO = :NRSEQCONTRATO
                   AND NRITEMOCONTRATO IS NULL
                   AND NRORG = :NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $return = $std->fetchAll();
        return $return;
    }
    
      /**
     * A função "deletarPaieFilhos" exclui  o local de Atendimento 
     * e os filhos.
     */
     
    public function deletarPaieFilhos($params){
       
        
        $sql = "DELETE FROM FCT_RELACESTRUTURACONT
                 WHERE NRORG      = :NRORG
                 AND NRSEQCONTRATO = :NRSEQCONTRATO
                 AND IDRELACESTRUTURA = 'CONTRATO'
                 AND (NRRELACESTRUTCONT = :NRESTRUTURAM
                 OR NRRELACESTRUTCONT IN (SELECT FCT_RELACESTRUTURACONT.NRRELACESTRUTCONT
                                                FROM FCT_RELACESTRUTURACONT
                                                JOIN HIERARQUIAORG 
                                                ON FCT_RELACESTRUTURACONT.NRESTRUTURAM = HIERARQUIAORG.NRESTRUTURAMREL
                                                AND FCT_RELACESTRUTURACONT.NRORG = HIERARQUIAORG.NRORG
                                                WHERE HIERARQUIAORG.NRORG  = :NRORG
                                                START WITH HIERARQUIAORG.NRESTRUTURAMREL = :NRESTRUTURAM
                                                CONNECT BY PRIOR HIERARQUIAORG.NRESTRUTURAMREL = HIERARQUIAORG.NRESTRUTURAMSUP))";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }


 /*=========================================================================================================================
  | À PARTIR DAQUI ESTÃO AS FUNÇÕES REFERENTES À INSERÇÃO DOS ITENS DE FORMAÇÃO DE PREÇO, QUE UTILIZAM O MESMO DATA SOURCE |
  =========================================================================================================================*/
    
     /**
      *  A função "insereItemContFormPreco" insere os valores da formação de preço
      *  no item do contrato.
     */
    
    public function insereItemContFormPreco($params){
        
        $sql = "INSERT INTO FCT_ITEMCONTRATO
                    (NRITEMOCONTRATO,  NRSEQCONTRATO,     CDSERVORCA,      NMITEMCONTRATO,  QTITEMCONTRATO, 
                     CDTPMEDIDASERV,   QTMEDIDACONTR,     VRUNITITEMCONT,  VRTOTITEMCONT,   IDMULTMEDVRTOT, 
                     CDHABILITSERV,    NRORG,             IDATIVO,         DTINCLUSAO,      NRORGINCLUSAO, 
                     CDOPERINCLUSAO,   CDTPMEDIDAVARIAVEL)
                VALUES
                    (:NRITEMOCONTRATO,  :NRSEQCONTRATO,                  :CDSERVORCA,               :NMITEMCONTRATO,          ROUND(:QTITEMCONTRATO, 2), 
                     :CDTPMEDIDASERV,   ROUND(NVL(:QTMEDIDACONTR,1), 2), ROUND(:VRUNITITEMCONT, 2), ROUND(:VRTOTITEMCONT, 2), NVL(:IDMULTMEDVRTOT, 'N'), 
                     :CDHABILITSERV,    :NRORG,                          'S',                       SYSDATE,                  :NRORG, 
                     :CDOPERADOR,       :CDTPMEDIDAVARIAVEL)";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
     /**
      * A função "somaContFormPreco" edita os itens do contrato que possuem serviços 
      * repetidos na formação dew preço.
     */
    
    public function somaContFormPreco($params, $medida){
       
        $sqlAUX = '';
         
        if ($medida == 'S'){ 
            $sqlAUX = "QTMEDIDACONTR = '1', ";
        }
        
        $sql = "UPDATE FCT_ITEMCONTRATO
                   SET VRUNITITEMCONT = ROUND(VRTOTITEMCONT + :VRTOTITEMCONT, 2),
                       QTITEMCONTRATO = '1', ".
                       $sqlAUX
                       ."
                       VRTOTITEMCONT = ROUND(VRTOTITEMCONT + :VRTOTITEMCONT, 2)
                 WHERE CDSERVORCA      = :CDSERVORCA
                   AND NRSEQCONTRATO   = :NRSEQCONTRATO
                   AND NRORG           = :NRORG
                   AND NRITEMOCONTRATO = :NRITEMOCONTRATO";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function getValorTotalItem($params) {
        $sql = "SELECT VRTOTITEMCONT
                FROM FCT_ITEMCONTRATO
                WHERE NRSEQCONTRATO    = :NRSEQCONTRATO
                   AND NRITEMOCONTRATO = :NRITEMOCONTRATO
                   AND NRORG           = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    } 
     /**
      * A função "retornaItensInseridos"  retorna os itens de formação de preço que foram inseridos no contrato
      */
    public function retornaItensInseridos ($codigos, $contrato ,$nrorg) {
        $sql = "SELECT NRITEMOCONTRATO, NRSEQCONTRATO, CDSERVORCA, NMITEMCONTRATO, VRTOTITEMCONT
                  FROM FCT_ITEMCONTRATO
                 WHERE NRITEMOCONTRATO  IN (".$codigos.")
                   AND NRSEQCONTRATO    = :NRSEQCONTRATO
                   AND NRORG            = :NRORG";
        
        $params = array("NRSEQCONTRATO" => $contrato,
                        "NRORG" => $nrorg);
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return  $std->fetchAll();
    }
    
    /**
     *  A função "insereRateio" insere os rateios dos itens do contrato
     */
    public function insereRateio(array $params) {
        $sql = "INSERT INTO FCT_RELACESTRUTURACONT
                    (NRRELACESTRUTCONT,     NRSEQCONTRATO,  NRITEMOCONTRATO,   NRESTRUTURAM,   VRITEMRATEIO, 
                    IDRELACESTRUTURA,       NRORG,          IDATIVO,           DTINCLUSAO,     NRORGINCLUSAO, 
                    CDOPERINCLUSAO,         VRPERCRATEIO)
                VALUES
                    (:NRRELACESTRUTCONT, :NRSEQCONTRATO,          :NRITEMOCONTRATO,   :NRESTRUTURAM,  :VRITEMRATEIO, 
                     :IDRELACESTRUTURA,  :NRORG,                  'S',                SYSDATE,        :NRORG, 
                     :CDOPERADOR,        ROUND(:VRPERCRATEIO, 2))";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
     /**
      *  A função "atualizaRateio" atualiza os rateios dos itens do contrato
     */
    
    public function atualizaRateio ($params){

        $sql = " UPDATE FCT_RELACESTRUTURACONT
                    SET VRITEMRATEIO     = ROUND(VRITEMRATEIO + :VRITEMRATEIO, 2)
                  WHERE NRSEQCONTRATO    = :NRSEQCONTRATO  
                    AND NRITEMOCONTRATO  = :NRITEMOCONTRATO   
                    AND NRESTRUTURAM     = :NRESTRUTURAM
                    AND IDRELACESTRUTURA = 'RATEIO'
                    AND NRORG            = :NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    

     /**
      *  A função "atualizaRateio" valida se ja existe um rateio criado 
      *  para o mesmo item e estrutura
     */
     
    public function buscaRateio ($params) {
        $sql = "SELECT COUNT (*) AS EXISTE
                 FROM FCT_RELACESTRUTURACONT
                WHERE NRSEQCONTRATO    = :NRSEQCONTRATO  
                  AND NRITEMOCONTRATO  = :NRITEMOCONTRATO   
                  AND NRESTRUTURAM     = :NRESTRUTURAM
                  AND IDRELACESTRUTURA = 'RATEIO'
                  AND NRORG            = :NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return  $std->fetchAll();
    }
    
    /**
     * Busca o parametro de geracao de rateio do contrato.
     */
    public function getIdGeraRateioContrato(array $params) {
        $sql = "SELECT IDGERARATEIO
                  FROM FCT_CAPAPARAMFATESC
                 WHERE NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                   AND NRORG           = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return  $std->fetchAll();
    }
    
    public function inserirParamFatItCont(array $params) {
        $sql = "INSERT INTO FCT_PARAMFATITCONT (
                    NRSEQFATUITCONT,    NRSEQCONTRATO,      NRITEMOCONTRATO,
                    CDPRODFATURA,       CDFILIALFATURA,     NRGRUPOFATURA,
                    NRPARCFATURA,       IDBLOQVRITMEDIC,    IDATIVO,
                    CDOPERINCLUSAO,     NRORG,              NRORGINCLUSAO,
                    DTINCLUSAO
                )
                VALUES (
                    :NRSEQFATUITCONT,   :NRSEQCONTRATO,     :NRITEMOCONTRATO,
                    :CDPRODFATURA,      :CDFILIALFATURA,    :NRGRUPOFATURA,
                    :NRPARCFATURA,      :IDBLOQVRITMEDIC,   :IDATIVO,
                    :CDOPERINCLUSAO,    :NRORG,             :NRORGINCLUSAO,
                    SYSDATE
                )";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    
    public function inserirRelacionamentoFormacaoPrecoContrato(array $params) {
        $sql = "INSERT INTO FCT_RELACFORMACAOCONT (
                    NRRELACFORMACAOCONT,    NRFORMPRECOSERVICO,     NRSEQCONTRATO,     NRITEMOCONTRATO,
                    IDATIVO,                NRORG,                  NRORGINCLUSAO,     DTINCLUSAO,
                    CDOPERINCLUSAO
                )
                VALUES (
                    :NRRELACFORMACAOCONT,   :NRFORMPRECOSERVICO,    :NRSEQCONTRATO,    :NRITEMOCONTRATO,
                    :IDATIVO,               :NRORG,                 :NRORG,            SYSDATE,
                    :CDOPERINCLUSAO
                )";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Busca os locais de atendimento e os itens de um contrato
    **/
    public function getItensEstruraContrato(array $params) {
        $sql = "SELECT DISTINCT FORMA.NRESTRUTURAM, LOCALM.CDCENTCUST,
                  LISTAGG(NRITEMOCONTRATO, ',') WITHIN GROUP (ORDER BY NRITEMOCONTRATO) AS NRITEMOCONTRATO
                FROM FCT_RELACFORMACAOCONT FORMCONT
                JOIN FCT_FORMPRECOSERVICO FORMA
                  ON FORMCONT.NRFORMPRECOSERVICO = FORMA.NRFORMPRECOSERVICO
                  AND FORMCONT.NRORG             = FORMA.NRORG
                LEFT JOIN ESTRUTURAM LOCALM
                  ON FORMA.NRESTRUTURAM = LOCALM.NRESTRUTURAM
                  AND FORMA.NRORG       = LOCALM.NRORG
                WHERE FORMCONT.NRSEQCONTRATO = :NRSEQCONTRATO
                  AND FORMCONT.NRORG         = :NRORG
                GROUP BY FORMA.NRESTRUTURAM, LOCALM.CDCENTCUST";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * Os dados de rateio de um item do contrato 
    **/
    public function getDadosRateioItemContrato(array $params) {
        $sql = "SELECT NRRATEIOITEMFAT,
                  NRSEQCONTRATO,
                  NRITEMOCONTRATO,
                  NRESTRUTURAM,
                  CDCENTCUST,
                  VRPERCENTRATEIO,
                  VRRATEADO
                FROM FCT_RATEIOITFATURACONT
                WHERE NRSEQCONTRATO   = :NRSEQCONTRATO
                  AND NRITEMOCONTRATO = :NRITEMOCONTRATO
                  AND NRORG           = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * Busca o percentual do rateio do item do contrato
    **/
    public function getPercentualRateioItem(array $params) {
        $sql = "SELECT (ITEM.VRTOTITEMCONT * 100 ) / VRTOTITEMCOMDESC AS VRPERCRATEIOITEM
                FROM FCT_NEGOCIACAOCONTRATO NEGO
                JOIN FCT_ITEMCONTRATO ITEM
                  ON NEGO.NRSEQCONTRATO = ITEM.NRSEQCONTRATO
                  AND NEGO.NRORG        = ITEM.NRORG
                WHERE NEGO.NRSEQCONTRATO   = :NRSEQCONTRATO
                  AND ITEM.NRITEMOCONTRATO = :NRITEMOCONTRATO
                  AND NEGO.NRORG           = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * retorna a contagem de rateios de uma proposta
    **/
    public function getRateiosItensContrato(array $params) {
        $sql = "SELECT DISTINCT ITEST.NRESTRUTURAM, 
                  LISTAGG(ITEM.NRITEMOCONTRATO, ',') WITHIN GROUP (ORDER BY ITEM.NRITEMOCONTRATO) AS NRITEMOCONTRATO
                FROM FCT_ITEMCONTRATO ITEM
                LEFT JOIN FCT_RELACESTRUTURACONT ITEST
                  ON ITEM.NRITEMOCONTRATO = ITEST.NRITEMOCONTRATO
                  AND ITEM.NRORG          = ITEST.NRORG
                WHERE ITEM.NRSEQCONTRATO    = :NRSEQCONTRATO
                  AND ITEM.NRORG            = :NRORG
                  AND (ITEST.IDRELACESTRUTURA IS NULL OR ITEST.IDRELACESTRUTURA = 'RATEIO')
                GROUP BY ITEST.NRESTRUTURAM";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * Busca a contagem de estruturas de um contrato
    **/
    public function getTotalEstrututasItensContrato(array $params) {
        $sql = "SELECT COUNT(NRESTRUTURAM) AS COUNTITENSRATEIO
                FROM (SELECT DISTINCT  NRESTRUTURAM
                      FROM FCT_RELACESTRUTURACONT ESTCONT
                      WHERE ESTCONT.NRSEQCONTRATO = :NRSEQCONTRATO
                        AND ESTCONT.NRORG         = :NRORG
                        AND ESTCONT.NRESTRUTURAM IS NOT NULL
                        AND ESTCONT.IDRELACESTRUTURA = 'RATEIO'
                        AND (SELECT COUNT(*) 
                             FROM FCT_ITEMCONTRATO ITEM
                             LEFT JOIN FCT_RELACESTRUTURACONT EST
                                ON ITEM.NRITEMOCONTRATO = EST.NRITEMOCONTRATO
                                AND ITEM.NRORG          = EST.NRORG
                             WHERE ITEM.NRSEQCONTRATO  = :NRSEQCONTRATO
                                AND ITEM.NRORG         = :NRORG
                                AND EST.NRESTRUTURAM IS NULL) = 0)";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * Deleta os rateios dos itens de um contrato
    **/
    public function deletarRateiosContrato(array $params) {
        $sql = "DELETE FROM FCT_RELACESTRUTURACONT
                WHERE NRSEQCONTRATO      = :NRSEQCONTRATO
                    AND IDRELACESTRUTURA = :IDRELACESTRUTURA
                    AND NRORG            = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Deleta os rateios de um item 
    **/
    public function deletarRateiosItemDoContrato(array $params) {
        $sql = "DELETE FROM FCT_RELACESTRUTURACONT
                WHERE NRSEQCONTRATO      = :NRSEQCONTRATO
                    AND NRITEMOCONTRATO  = :NRITEMOCONTRATO
                    AND NRORG            = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * busca as estruturas sem centro de custo
    **/
    public function getEstruturasSemCentroCusto($estruturas) {
        $sql = "SELECT NRESTRUTURAM
                FROM ESTRUTURAM
                WHERE NRESTRUTURAM IN ($estruturas)
                    AND CDCENTCUST IS NULL";
        $std = $this->connection->prepare($sql);
        $std->execute();
        return $std->fetchAll();
    }
}
?>