<?php
namespace Teknisa\SM\Service;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Teknisa\SM\Factory\Sm00004000ContratoLocalAtendFactory;
use Teknisa\SM\Factory\Sm00001000ItemLancamentoPropostaFactory;

class Sm00004000ContratoLocalAtendService
{
    
    protected $connection;
    protected $environment;
    protected $factoryLocalAtendCont;
    protected $itensPropostaFactory;
    
    public function __construct (EntityManager $entityManager, Sm00004000ContratoLocalAtendFactory $factoryLocalAtendCont, Environment $environment,  NewCode $newCode, Sm00001000ItemLancamentoPropostaFactory $itensPropostaFactory){
        
        $this->connection            = $entityManager->getConnection();
        $this->factoryLocalAtendCont = $factoryLocalAtendCont;
        $this->environment           = $environment;
        $this->newCode               = $newCode;
        $this->itensPropostaFactory  = $itensPropostaFactory;

    }
    
    /**
     * A função "validaLocalAtend" verifica se o local de Atendimento 
     * que está sendo relacionado já está relacionado na tabela.
     */
     
    public function validaLocalAtend($nrEstruturam, $nrSeqContrato){
    
        $params = array(
            "NRORG"          => $this->environment->getNrOrg(),
            "NRSEQCONTRATO"  => $nrSeqContrato,
            "NRESTRUTURAM"   => $nrEstruturam
        );
        
        $return = $this->factoryLocalAtendCont->validaLocalAtend($params);
        
        return $return[0]["VALIDA"] != 0;
        
    }
    
    /**
     * A função "deletarPaieFilhos" exclui  o local de Atendimento 
     * e os filhos.
     */
    
    public function deletarPaieFilhos($nrEstruturam, $nrSeqContrato){
    
        $params = array(
            "NRESTRUTURAM"   => $nrEstruturam,
            "NRSEQCONTRATO"  => $nrSeqContrato,
            "NRORG"          => $this->environment->getNrOrg()
        );
        
        $this->factoryLocalAtendCont->deletarPaieFilhos($params);
        
    }
    
    
    /*=========================================================================================================================
     | À PARTIR DAQUI ESTÃO AS FUNÇÕES REFERENTES À INSERÇÃO DOS ITENS DE FORMAÇÃO DE PREÇO, QUE UTILIZAM O MESMO DATA SOURCE |
     =========================================================================================================================*/
    
    /**
     * A função "insereItemFormPrec" prepara o array para inserção dos itens de contrato 
     * e centraliza as funções de inserção com soma ou separadamente.  
     */
    
    public function insereItemFormPrec($servicos, $soma, $contrato, $nrtipoorcamento) {
        $nrorg      = $this->environment->getNrOrg();
        $operador   = $this->environment->getCdOperador();
        $geraRateio = $this->verificaGeraRateioContrato($nrtipoorcamento, $nrorg);
        $itensInseridos = null;
        
        if ($soma == true) {
            $itensInseridos = $this->insereItFormSoma($servicos, $contrato, $nrorg, $operador, $geraRateio, $nrtipoorcamento);
        } else {
            $itensInseridos = $this->insereItForm($servicos, $contrato, $nrorg, $operador, $geraRateio, $nrtipoorcamento);
        }
        
        return $itensInseridos;
    }

    /**
     * Insere os itens de contrato quando escolhida 
     * a opção "Somar os Valores".
     */
    public function insereItFormSoma($itens, $contrato, $nrorg, $operador, $geraRateio, $nrtipoorcamento) {
        $servicos        = array();
        $valorTotalItens = array();
        $itensInseridos  = array();
        $this->connection->beginTransaction();
        
        try {
            foreach ($itens as &$item) {
                $cdservorca = $item["CDSERVORCA"];
                if (array_key_exists($cdservorca, $servicos)) {
                    
                    $item["NRITEMOCONTRATO"] = $servicos[$cdservorca];
                    if (isset($valorTotalItens[$cdservorca])) {
                        $valorTotalItens[$cdservorca] += $item["VRTOTAL"];
                    }
                    
                    $params = array (
                        "NRITEMOCONTRATO" => $item["NRITEMOCONTRATO"],
                        "VRTOTITEMCONT"   => $item["VRTOTAL"],
                        "CDSERVORCA"      => $cdservorca,
                        "NRSEQCONTRATO"   => $contrato, 
                        "NRORG"           => $nrorg
                    );
                    $this->factoryLocalAtendCont->somaContFormPreco($params, $item["IDUSAMEDIDA"]);
                } else {
                    $novoCodigo = $this->newCode->getCodeByTableName("FCT_ITEMCONTRATO", $nrorg, 1, 10, true);
                    $item["NRITEMOCONTRATO"] = $novoCodigo;
                    $valorTotalItens[$cdservorca] = $item["VRTOTAL"];
                    $params = array(
                        "NRITEMOCONTRATO"    => $novoCodigo, 
                        "NRSEQCONTRATO"      => $contrato, 
                        "CDSERVORCA"         => $item["CDSERVORCA"],
                        "NMITEMCONTRATO"     => $item["NMSERVORCA"]." - ".$item["NMELEMENTO"],
                        "QTITEMCONTRATO"     => $item["QTCALCULO"],
                        "CDTPMEDIDASERV"     => $item["CDTPMEDIDASERV"],
                        "QTMEDIDACONTR"      => $item["QTMEDIDACALC"],
                        "VRUNITITEMCONT"     => $item["VRUNITARIO"],
                        "VRTOTITEMCONT"      => $item["VRTOTAL"],
                        "IDMULTMEDVRTOT"     => $item["IDUSAMEDIDA"],
                        "CDHABILITSERV"      => $item["CDHABILITSERV"],
                        "NRORG"              => $nrorg,
                        "CDOPERADOR"         => $operador,
                        "CDTPMEDIDAVARIAVEL" => null
                    );
                    
                    $this->factoryLocalAtendCont->insereItemContFormPreco($params);
                    $this->inserirParamFatItCont($novoCodigo, $contrato, $nrorg, $operador);
                    $this->inserirRelacionamentoFormacaoPrecoContrato($item["NRFORMPRECOSERVICO"], $novoCodigo, $contrato, $nrorg, $operador);
                    $servicos[$item["CDSERVORCA"]] = $novoCodigo;
                    $itensInseridos[] = array_merge($params, array("IDELEMENTO" => $item["IDELEMENTO"], "NRTIPOORCAMENTO" => $nrtipoorcamento));
                }
            }
            unset($item);
            
            if ($geraRateio) 
                $this->geraRateios($itens, $contrato, true, $valorTotalItens);
            
            $this->connection->commit();
            
            foreach($itensInseridos as &$item) {
                $params = array (
                    "NRITEMOCONTRATO" => $item["NRITEMOCONTRATO"],
                    "NRSEQCONTRATO"   => $contrato, 
                    "NRORG"           => $nrorg
                );
                
                $totalItem = $this->factoryLocalAtendCont->getValorTotalItem($params);
                $totalItem = $totalItem[0]["VRTOTITEMCONT"];
                
                $item["VRTOTITEMCONT"] = $totalItem;
            }
            return $itensInseridos;
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    /**
     * Insere os itens de contrato quando escolhida
     * a opção "incluir Separadamente" ou quando não possuem serviços iguais.
     */
    public function insereItForm($itens, $contrato, $nrorg, $operador, $geraRateio, $nrtipoorcamento) {
        $this->connection->beginTransaction();
        //$nritemocontrato = array();
        $itensInseridos = array();
        
        try {
            foreach ($itens as &$item) {
                $novoCodigo = $this->newCode->getCodeByTableName("FCT_ITEMCONTRATO", $nrorg, 1, 10, true);
                $item["NRITEMOCONTRATO"] = $novoCodigo;
                 
                $params = array(
                    "NRITEMOCONTRATO"    => $novoCodigo, 
                    "NRSEQCONTRATO"      => $contrato, 
                    "CDSERVORCA"         => $item["CDSERVORCA"],
                    "NMITEMCONTRATO"     => $item["NMSERVORCA"]." - ".$item["NMELEMENTO"],
                    "QTITEMCONTRATO"     => $item["QTCALCULO"],
                    "CDTPMEDIDASERV"     => $item["CDTPMEDIDASERV"],
                    "QTMEDIDACONTR"      => is_null($item["QTMEDIDACALC"]) ? 1 : $item["QTMEDIDACALC"],
                    "VRUNITITEMCONT"     => $item["VRUNITARIO"],
                    "VRTOTITEMCONT"      => $item["VRTOTAL"],
                    "IDMULTMEDVRTOT"     => $item["IDUSAMEDIDA"],
                    "CDHABILITSERV"      => $item["CDHABILITSERV"],
                    "NRORG"              => $nrorg,
                    "CDOPERADOR"         => $operador,
                    "CDTPMEDIDAVARIAVEL" => null
                );
                
                $medida = $this->getMedidaVariavelEscopo($nrtipoorcamento, $item["CDSERVORCA"], $nrorg);
                if (!empty($medida)) {
                    $params["CDTPMEDIDASERV"]     = $medida["CDTPMEDIDAVARIAVEL"];
                    $params["CDTPMEDIDAVARIAVEL"] = $medida["CDTPMEDIDAVARIAVEL"];
                    $params["IDMULTMEDVRTOT"]     = "S";
                }
                
                $this->factoryLocalAtendCont->insereItemContFormPreco($params);
                
                if (!empty($medida)) {
                    $params["INSERIR_MEDIDA"] = true;
                }
                
                $this->inserirParamFatItCont($novoCodigo, $contrato, $nrorg, $operador);
                $this->inserirRelacionamentoFormacaoPrecoContrato($item["NRFORMPRECOSERVICO"], $novoCodigo, $contrato, $nrorg, $operador);
                $itensInseridos[] = array_merge($params, array("IDELEMENTO" => $item["IDELEMENTO"], "NRTIPOORCAMENTO" => $nrtipoorcamento));
            }
            unset($item);
            
            if ($geraRateio) {
                $this->geraRateios($itens, $contrato);
            }
            
            $this->connection->commit();
            return $itensInseridos;
        }
        catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    public function getMedidaVariavelEscopo($nrtipoorcamento, $cdservorca, $nrorg) {
        $params = array(
            "NRTIPOORCAMENTO" => $nrtipoorcamento,
            "CDSERVORCA"      => $cdservorca,
            "NRORG"           => $nrorg
        );

        $dataset = $this->itensPropostaFactory->getMedidaVariavelEscopo($params);
        return (count($dataset) > 0) ? $dataset[0] : null;
    }
    
    /**
     * Insere os rateios para os itens do contrato
     */
    public function geraRateios($itens, $contrato, $soma = false, $valorTotalItens = null) {
        $nrorg    = $this->environment->getNrOrg();
        $operador = $this->environment->getCdOperador();
        
        $tpestrutura = "RATEIO";
        $estruturas  = array();
        foreach($itens as $item) 
            array_push($estruturas, $item["NRESTRUTURAM"]);
        
        $estruturas        = implode(",", $estruturas);
        $centrocustos      = $this->factoryLocalAtendCont->getEstruturasSemCentroCusto($estruturas);
        $possuiCentroCusto = count($centrocustos) == 0;
        
        if (!$possuiCentroCusto) {
            $this->atualizarTpEstruturaRateiosInseridos($contrato, $nrorg);
        } else {
            foreach ($itens as $item) {
                $cdservorca = $item["CDSERVORCA"];
                $estrutura  = $item["NRESTRUTURAM"];
                $vrRat      = $item["VRTOTAL"];
                $nrItem     = $item["NRITEMOCONTRATO"];
                
                if ($estrutura != null) {
                    if (!$soma) {
                        $percentRateio = 100;
                    } else {
                        $valorTotalItem = $valorTotalItens[$cdservorca];
                        $percentRateio  = $vrRat / $valorTotalItem * 100;
                    }
                    
                    $novoCodigo = $this->newCode->getCodeByTableName("NRRELACESTRUTCONT", $nrorg, 1, 10, true);
                    $params = array(
                        "NRRELACESTRUTCONT" => $novoCodigo, 
                        "NRSEQCONTRATO"     => $contrato, 
                        "NRITEMOCONTRATO"   => $nrItem, 
                        "NRESTRUTURAM"      => $estrutura,
                        "VRITEMRATEIO"      => $vrRat, 
                        "NRORG"             => $nrorg,
                        "CDOPERADOR"        => $operador,
                        "VRPERCRATEIO"      => $percentRateio,
                        "IDRELACESTRUTURA"  => $tpestrutura
                    );
                    
                    $this->factoryLocalAtendCont->insereRateio($params);
                }
            }
        }
    }
    /**
     * Atualiza o tipo de estrutura dos rateios de uma proposta
    **/
    public function atualizarTpEstruturaRateiosInseridos($contrato, $nrorg) {
        $params = array(
            "NRSEQCONTRATO"    => $contrato,
            "IDRELACESTRUTURA" => 'RATEIO',
            "NRORG"            => $nrorg
        );
        $this->factoryLocalAtendCont->deletarRateiosContrato($params);
    }
    
    /**
     * Busca os rateios ligados ao contrato
    **/
    public function getRateiosItensContrato($contrato, $nrorg) {
        $params = array(
            "NRSEQCONTRATO" => $contrato,
            "NRORG"         => $nrorg
        );
        return $this->factoryLocalAtendCont->getRateiosItensContrato($params);
    }
    
    /**
     *  A função "buscaRateio" verifica se ja existe o rateio para o item e estrutura
     */
    public function buscaRateio($contrato, $nrItem, $estrutura){
        $nrorg = $this->environment->getNrOrg();
        
        $params = array("NRSEQCONTRATO"     => $contrato, 
                        "NRITEMOCONTRATO"   => $nrItem, 
                        "NRESTRUTURAM"      => $estrutura, 
                        "NRORG"             => $nrorg
        );
        
        $return = $this->factoryLocalAtendCont->buscaRateio($params);
        return $return[0]["EXISTE"] != 0;
        
    }
    
    /**
     * Verifica se o contrato gera rateio. Retorna true ou false.
     */
    public function verificaGeraRateioContrato($nrtipoorcamento, $nrorg) {
        $params = array(
            "NRTIPOORCAMENTO" => $nrtipoorcamento,
            "NRORG"           => $nrorg
        );
        
        $dataset = $this->factoryLocalAtendCont->getIdGeraRateioContrato($params);
        return (empty($dataset) ? null : $dataset[0]["IDGERARATEIO"] == "S");
    }
    
    public function inserirParamFatItCont($nritemocontrato, $contrato, $nrorg, $operador) {
        $nrseqfatuitcont = $this->newCode->getCodeByTableName("NRSEQFATUITCONT", $nrorg, 1, 10, true);
        $params = array(
            "NRSEQFATUITCONT" => $nrseqfatuitcont,
            "NRSEQCONTRATO"   => $contrato,
            "NRITEMOCONTRATO" => $nritemocontrato,
            "CDPRODFATURA"    => null,
            "CDFILIALFATURA"  => null,
            "NRGRUPOFATURA"   => null,
            "NRPARCFATURA"    => null,
            "IDBLOQVRITMEDIC" => "N",
            "IDATIVO"         => "S",
            "CDOPERINCLUSAO"  => $operador,
            "NRORG"           => $nrorg,
            "NRORGINCLUSAO"   => $nrorg
        );
        
        $this->factoryLocalAtendCont->inserirParamFatItCont($params);
    }
    
    public function inserirRelacionamentoFormacaoPrecoContrato($nrformprecoservico, $nritemocontrato, $nrseqcontrato, $nrorg, $cdoperador) {
        $params = array(
            "NRRELACFORMACAOCONT" => $this->newCode->getCodeByTableName("FCT_RELACFORMACAOCONT", $nrorg, 1, 10, true),
            "NRFORMPRECOSERVICO" => $nrformprecoservico,
            "NRSEQCONTRATO"      => $nrseqcontrato,
            "NRITEMOCONTRATO"    => $nritemocontrato,
            "IDATIVO"            => "S",
            "NRORG"              => $nrorg,
            "CDOPERINCLUSAO"     => $cdoperador
        );
        
        $this->factoryLocalAtendCont->inserirRelacionamentoFormacaoPrecoContrato($params);
    }
    /**
     * retorna as estruturas ligadas ao rateio 
    **/
    public function validarDadosRateioContrato($nrseqcontrato, $nrorg, $newEstruturas, $nrtipoorcamento) {
        $paramsFaturmento = $this->verificaGeraRateioContrato($nrtipoorcamento, $nrorg);
        if($paramsFaturmento) {
            if(!is_array($newEstruturas)) $newEstruturas = array($newEstruturas);
            
            $estruturasAtuais = implode(",", $newEstruturas);
            if($estruturasAtuais) {
                $centrocustos = $this->factoryLocalAtendCont->getEstruturasSemCentroCusto($estruturasAtuais);
                if(count($centrocustos) > 0)
                    return "SEM_CENTROCUST";
            }
            
            $params = array(
                "NRSEQCONTRATO" => $nrseqcontrato,
                "NRORG"         => $nrorg
            );
            
            $rateiosProposta = $this->factoryLocalAtendCont->getTotalEstrututasItensContrato($params);
            if($rateiosProposta[0]["COUNTITENSRATEIO"] > 0) {
                if(count($newEstruturas) > 0 && in_array(null, $newEstruturas))
                    return "RATEIO_SERA_DELETADO";
            }
        }
        
        return null;
    }
}

?>