<?php
namespace Teknisa\SM\Service;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment;
use Teknisa\SM\Util\Procedure\FctAtualizaDocumento;
use Teknisa\SM\Factory\Sm00004006DocumentoContratoFactory;
use Teknisa\SM\Util\Utilities;
use Zeedhi\Framework\Remote\FileUploader\Server;
use Teknisa\SM\Service\SmDocumentosService;

class Sm00004006DocumentoContratoService {
    
    protected $environment;
    protected $connection;
    protected $documentoContratoFactory;
    protected $fileServerBaseUrl;
    protected $zhFileUploader;
    protected $smDocumentosService;
    
    public function __construct(EntityManager $entityManager, Environment $environment, Sm00004006DocumentoContratoFactory $documentoContratoFactory, $fileServerBaseUrl, Server $zhFileUploader, SmDocumentosService $smDocumentosService) {
        $this->connection               = $entityManager->getConnection();    
        $this->environment              = $environment;
        $this->documentoContratoFactory = $documentoContratoFactory;
        $this->fileServerBaseUrl        = $fileServerBaseUrl;
        $this->zhFileUploader           = $zhFileUploader;
        $this->smDocumentosService      = $smDocumentosService;
    }
    
    /**
     * A função "atualizaDocumento" Atualiza o documento de acordo com o modelo encontrado.
     */
     
    public function atualizaDocumento($nrseqcontdoc, $nrseqmoddocfct) {
        $this->connection->beginTransaction();
        try {
            $procedure = new FctAtualizaDocumento($this->connection);
            
            $nrorg =  $this->environment->getNrOrg();
            $operador =  $this->environment->getCdOperador();
            
            $procedure->executar(("_".$nrseqcontdoc."_"), 'C', $nrseqmoddocfct, $nrorg, $operador);
            
            $documento = $this->getDocumentoContrato($nrseqcontdoc, $nrorg);
            $nomeArquivo   = $documento["NMARQUIVO"];
            $tipo          = $documento["DSTIPO"];
            $tamanhoBytes  = $documento["VRTAMANHO"] * 1024;
            $urlFileServer = $documento["DSCAMINHOARQ"];
            
            if (!empty($urlFileServer)) {
                //$conteudo = @file_get_contents($this->fileServerBaseUrl . $urlFileServer);
                $conteudo = $this->smDocumentosService->getDocumentoFileServer($this->fileServerBaseUrl . $urlFileServer, $nrorg);
                if ($conteudo !== false) {
                    $file = Utilities::uploadFileServer($this->zhFileUploader, "anexo_documento_contrato", $nomeArquivo, $conteudo, $tamanhoBytes, $tipo);
                    $caminhoNovo = $file["path"];
                    
                    $params = array(
                        "DSCAMINHOARQ" => $caminhoNovo,
                        "CDOPERULTATU" => $operador,
                        "NRORGULTATU"  => $nrorg,
                        "NRSEQCONTDOC" => $nrseqcontdoc,
                        "NRORG"        => $nrorg
                    );
                    
                    $this->documentoContratoFactory->atualizarCaminhoDocumentoContrato($params);
                }
            }
            
            $this->connection->commit();
        }
        catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    public function limparAnexoDoContrato($nrseqcontdoc, $nrorg) {
        $params = array(
            "NRSEQCONTDOC" => $nrseqcontdoc,
            "NRORG"        => $nrorg
        );
        
        $this->documentoContratoFactory->limparAnexoDoContrato($params);
    }
    
    /**
     * Retorna o registro da tabela de documento do contrato.
     */
    public function getDocumentoContrato($nrseqcontdoc, $nrorg) {
        $params = array(
            "NRSEQCONTDOC" => $nrseqcontdoc,
            "NRORG"        => $nrorg
        );
        
        $dataset = $this->documentoContratoFactory->getDocumentoContrato($params);
        return count($dataset) > 0 ? $dataset[0] : [];
    }
    
    /**
     * Baixa do fileServe o arquivo de documento e salva em uma pasta temporária
    **/
    public function downloadDocFileServer($nrseqcontdoc, $nrorg) {
        $result    = array();
        $documento = $this->getDocumentoContrato($nrseqcontdoc, $nrorg);
        if(count($documento) > 0) {
            $microtime = microtime(true);
            $microtime = str_replace(".", "_", $microtime);
            
            $fileDir   = $this->fileServerBaseUrl . $documento["DSCAMINHOARQ"];
            $fileName  = $microtime . "_" . str_replace(",", "_", $documento["NMARQUIVO"]);
            
            $fileDirTemp = $this->smDocumentosService->downloadDocFileServer($fileDir, $fileName, $nrorg);
            $fileName    = base64_encode($fileName);
            $result = "/downloadArquivo?requestType=Row&row[FILENAME]=" . $fileName . "&row[FILETYPE]=" . $documento["DSTIPO"];
        }
        
        return $result;
    }
}

?>