<?php

namespace Teknisa\SM\Service;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\ConnectionCustom;
//use Teknisa\SM\Service\Sm00001000LancamentoPropostaService;
use Teknisa\SM\Factory\Sm00012000MedicaoFaturamentoFactory;
use Teknisa\SM\Factory\Sm00004000ContratoFactory;
use Teknisa\SM\Util\Procedure\FctGerarParcelasApuracao;
use Teknisa\SM\Util\Procedure\FctCancelarMedicao;
use Teknisa\SM\Util\Procedure\FctCancelarApuraMedicao;
use Teknisa\SM\Util\Utilities;


class Sm00012000MedicaoFaturamentoService {
    
    protected $connection;
    protected $environment;
    protected $connectioncustom;
    //protected $propostaService;
    protected $medicaoFactory;
    protected $contratoFactory;
    
    public function __construct(EntityManager $entityManager, Environment $environment, ConnectionCustom $connectioncustom,/* Sm00001000LancamentoPropostaService $propostaService,*/ Sm00012000MedicaoFaturamentoFactory $medicaoFactory, Sm00004000ContratoFactory $contratoFactory) {
        $this->connection       = $entityManager->getConnection();
        $this->environment      = $environment;
        $this->entityManager    = $entityManager;
        $this->connectioncustom = $connectioncustom;
        //$this->propostaService  = $propostaService;
        $this->medicaoFactory   = $medicaoFactory;
        $this->contratoFactory  = $contratoFactory;
    }
    
    /**
    * Faz o cálculo dos itens de acordo com a medida variável
    */
    public function recalcularItensMedicao($nrmedicaofatura, $nrseqcontrato, $nrorg = null) {
        $nrorg      = $nrorg ? $nrorg : $this->environment->getNrorg();
        $cdoperador = $this->environment->getCdOperador();
        
        $dadosMedicao = $this->getDadosMedicao($nrmedicaofatura, $nrorg);

        $itensMedicao = $this->getItensMedicao($nrmedicaofatura, $nrorg);
        $itensComValorNegociado = array();
        $somaItensMedicao = 0;
        
        foreach ($itensMedicao as $itMed) {
            $nritmedicaofatura = $itMed["NRITMEDICAOFATURA"];
            $nrmedicaofatura   = $itMed["NRMEDICAOFATURA"];
            $nritemcontrato    = $itMed["NRITEMOCONTRATO"];
            $cdservorca        = $itMed["CDSERVORCA"];

            $params = array(
                "P_NRMEDICAOFATURA" => $nrmedicaofatura,
                "P_NRSEQCONTRATO"   => $nrseqcontrato,
                "P_NRORG"           => $nrorg,
                "P_CDSERVORCA"      => $cdservorca
            );
            
            $servicos = $this->medicaoFactory->getItensContrato($params);
            
            $valorSomaElementos = 0;
            $valorTotalNegociado = null;
            
            $valorElemento = 0;
            $valorElementoComDesconto = 0;
            $valorSomaElementosComDesconto = 0;
            
            foreach ($servicos as $serv) {
                if ($serv["QTMEDIDAMEDICAO"] == 0) {
                    $valorElemento = 0;
                }
                else if (is_null($serv["VRPRECOVARIACAO"])) {
                    $valorElemento = $serv["VRUNITITEMORCA"];
                }
                else if ($serv["VRPRECOVARIACAOVEND"] == 0 || is_null($serv["VRPRECOVARIACAOVEND"])) {
                    $valorElemento = $serv["VRPRECOVARIACAO"];
                }
                else if(is_null($serv["IDELEMENTORAIZ"]) && $serv["VRUNITITEMORCA"] == $serv["VRTOTITEMCONT"]){
                    $valorElemento = $serv["VRPRECOVARIACAOVEND"];
                }
                else {
                    $valorElemento = $serv["VRUNITITEMORCA"] / $serv["VRPRECOVARIACAOVEND"] * $serv["VRPRECOVARIACAO"];
                }

                if ($serv["IDMULTIPVARIACAO"] === "Sim" || is_null($serv["IDELEMENTORAIZ"])) {
                    $valorElemento *= $serv["QTMEDIDAMEDICAO"];
                }
                else {
                    $valorElemento *= $serv["QTMEDIDACONTR"];
                }

                $valorElemento = round($valorElemento, 2);
                
                // Calcula o desconto do elemento
                $valorElementoComDesconto = $valorElemento;
                if (!is_null($serv["VRDESCITEM"]) && $serv["VRDESCITEM"] > 0) {
                    $idtpdesconto   = $serv["IDTPDESCONTO"];
                    $vrdescitem     = $serv["VRDESCITEM"];
                    $vrpercdescitem = $serv["VRPERCDESCITEM"];
                    
                    if ($idtpdesconto == "V") {
                        $valorElementoComDesconto -= $vrdescitem;
                    }
                    else {
                        $valorElementoComDesconto -= ($valorElementoComDesconto * $vrpercdescitem / 100);
                    }
                    
                    $valorElementoComDesconto = round($valorElementoComDesconto, 2);
                }
                
                /* Verifica se existe uma negociação para este elemento que resultará na alteracao do seu valor - INICIO */
                $valorNegociado = null;
                $nrparcelasnegcontr = null;
                
                $negociacaoElemento = $this->getNegociacaoContratoParaElemento($dadosMedicao["NRSEQCONTRATO"], $dadosMedicao["DTINICOMPET"], $dadosMedicao["DTFINCOMPET"], $dadosMedicao["NRPARCFATURA"], $serv["NRITEMOCONTRATO"], $nrorg);
                if ($negociacaoElemento) {
                    $valorNegociado = $this->calculaValorNegociado($valorElemento, $serv["IDMULTIPVARIACAO"], $serv["QTMEDIDAMEDICAO"], $negociacaoElemento);
                    $valorTotalNegociado += $valorNegociado;
                    
                    $nrparcelasnegcontr = $negociacaoElemento["NRPARCELASNEGCONTR"];
                }
                else {
                    $valorTotalNegociado += $valorElementoComDesconto;
                }
                /* Verifica se existe uma negociação para este elemento que resultará na alteracao do seu valor - FIM */
                
                //Atualiza a medida variavel com o valor calculado e o valor negociado
                $this->atualizarMedidaVariavel('S', $valorElemento, $valorElementoComDesconto, $valorNegociado, $nrparcelasnegcontr, $nrmedicaofatura, $serv["IDELEMENTORAIZ"], $serv["CDSERVORCA"], $serv["CDTPMEDIDAVARIAVEL"], $nrorg, $cdoperador);
                
                $valorSomaElementos += $valorElemento;
                $valorSomaElementosComDesconto += $valorElementoComDesconto;
            }
            
            $valorSomaElementos = round($valorSomaElementos, 2);
            $valorSomaElementosComDesconto = round($valorSomaElementosComDesconto, 2);
            
            if ($valorTotalNegociado == $valorSomaElementosComDesconto) {
                $valorTotalNegociado = null;
            }
            
            //Não foi calculado nenhum desconto, então os valores com e sem desconto são iguais (cálculo mudou de lugar)
            $vrtotitemcomdesc = $valorSomaElementosComDesconto;
            
            $nrparcelasnegcontr = null;
            /* Verifica se existe uma negociação para este item (serviço) que resultará na alteracao do seu valor - INICIO */
            if (is_null($valorTotalNegociado)) {
                $negociacaoServico = $this->getNegociacaoContratoParaServico($dadosMedicao["NRSEQCONTRATO"], $dadosMedicao["DTINICOMPET"], $dadosMedicao["DTFINCOMPET"], $dadosMedicao["NRPARCFATURA"], $itMed["NRITEMOCONTRATO"], $nrorg);
                if ($negociacaoServico) {
                    $valorTotalNegociado = $this->calculaValorNegociado($valorSomaElementosComDesconto, null, null, $negociacaoServico);
                    
                    $itensComValorNegociado[] = $valorTotalNegociado;
                    $nrparcelasnegcontr = $negociacaoServico["NRPARCELASNEGCONTR"];
                }
            }
            /* Verifica se existe uma negociação para este item (serviço) que resultará na alteracao do seu valor - FIM */
            
            
            // Valida e calcula valor proporcional - INICIO
            $proporcional = $this->validaCalculaProporcional($nrmedicaofatura, $nrseqcontrato, $valorSomaElementosComDesconto, $vrtotitemcomdesc, $valorTotalNegociado, $nrorg);
            $valorSomaElementosComDesconto = $proporcional["VRTOTALITEM"];
            $vrtotitemcomdesc              = $proporcional["VRTOTALITDESC"];
            $valorTotalNegociado           = $proporcional["VRNEGOCIADO"];
            // Valida e calcula valor proporcional - FIM
            
            $this->updateItensMedicao($valorSomaElementosComDesconto, $vrtotitemcomdesc, $valorTotalNegociado, $nrparcelasnegcontr, '3', $nritmedicaofatura, $nrmedicaofatura, $nritemcontrato, $cdservorca, $nrorg, $cdoperador);
            
            $somaItensMedicao += $valorSomaElementosComDesconto;
        }
        
        //Verifica se tem desconto na negociação de valores para aplicar no itens da medição
        $dadosNegociacao = $this->getDadosNegociacaoMedicao($nrmedicaofatura, $dadosMedicao["DTINICOMPET"], $nrorg);
        
        if (!is_null($dadosNegociacao["VRDESCITEM"]) && $dadosNegociacao["VRDESCITEM"] > 0 && $somaItensMedicao > 0) {
            $idtpdesconto   = $dadosNegociacao["IDTPDESCONTO"];
            $vrdescitem     = $dadosNegociacao["VRDESCITEM"];
            $vrpercdescitem = $dadosNegociacao["VRPERCDESCITEM"];
            
            $this->calculaDescontoItemESalva($somaItensMedicao, $idtpdesconto, $vrdescitem, $vrpercdescitem, $nrmedicaofatura, $nrorg, $cdoperador);
        }
        
        //Verifica se tem desconto por período na negociação de valores para aplicar no itens da medição
        if (!is_null($dadosNegociacao["VRDESCITEMPORPERIODO"]) && $dadosNegociacao["VRDESCITEMPORPERIODO"] > 0 && $somaItensMedicao > 0) {
            $idtpdesconto   = $dadosNegociacao["IDTPDESCPORPERIODO"];
            $vrdescitem     = $dadosNegociacao["VRDESCITEMPORPERIODO"];
            $vrpercdescitem = $dadosNegociacao["VRPERCDESCPERIODO"];
            
            $this->calculaDescontoItemESalva($somaItensMedicao, $idtpdesconto, $vrdescitem, $vrpercdescitem, $nrmedicaofatura, $nrorg, $cdoperador);
        }
        
        $this->updateMedicao($nrmedicaofatura, $nrorg, $dadosMedicao);
    }
    
    private function calculaDescontoItemESalva($somaItensMedicao, $idtpdesconto, $vrdescitem, $vrpercdescitem, $nrmedicaofatura, $nrorg, $cdoperador) {
        $params = array(
            "NRMEDICAOFATURA" => $nrmedicaofatura,
            "NRORG"           => $nrorg
        );
        $itensMedicao = $this->medicaoFactory->getItensMedicaoComSoma($params);
        
        $somatorioDesconto = 0;
        $qtdItens = count($itensMedicao);
        
        foreach ($itensMedicao as $key => $itMed) {
        
            $vritemcont = $itMed["VRITEMCONT"];
            
            if ($idtpdesconto == "V") {
                if ($vrdescitem >= $somaItensMedicao) {
                    $valorDescontoNegociacao = $vritemcont;
                }
                else if ($key == $qtdItens-1) {
                    $valorDescontoNegociacao = $vrdescitem - $somatorioDesconto;
                }
                else {
                    $perc = $vritemcont / $somaItensMedicao * 100;
                    $valorDescontoNegociacao = round(($vrdescitem * $perc / 100), 2);
                    $somatorioDesconto += $valorDescontoNegociacao;
                }
            }
            else {
                $valorDescontoNegociacao = round(($vritemcont * $vrpercdescitem / 100), 2);
            }
            
            $vritemcont = $vritemcont - $valorDescontoNegociacao;
            
            $this->updateItensMedicao($itMed["VRTOTITEMCONT"], $vritemcont, $itMed["VRNEGCONTR"], $itMed["NRPARCELASNEGCONTR"], $itMed["IDSITUACAO"], $itMed["NRITMEDICAOFATURA"], $itMed["NRMEDICAOFATURA"], $itMed["NRITEMOCONTRATO"], $itMed["CDSERVORCA"], $nrorg, $cdoperador);
        }
    }
    
    /* Valida o prazo da parcela de medição e faz o calculo proporcional caso seja necessário */
    public function validaCalculaProporcional($nrmedicaofatura, $nrseqcontrato, $valorTotalItem, $vrtotitemcomdesc, $valorTotalNegociado, $nrorg) {
        
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg
        );

        $paramFatInc = $this->medicaoFactory->recebeParamFatIncompleto($params);
        
        if (!empty($paramFatInc) && count($paramFatInc) > 0) {
        
            $fatuIcomp = $paramFatInc[0]["IDTPCALCPARCINC"];
            
            /* Faz o cálculo do valor proporcional aos dias do período */
            if ($fatuIcomp == '1') {
            
                $params = array(
                    "NRMEDICAOFATURA" => $nrmedicaofatura,
                    "NRORG"           => $nrorg
                );
        
                $prazos = $this->medicaoFactory->validaProporcional($params);
                
                $prazo_Integral = $prazos[0]["PRAZOINT"];
                $prazo_medicao = $prazos[0]["PRAZO"];
                
                if ($prazo_Integral == $prazo_medicao){
                    $retorno = array(
                        "VRTOTALITEM"   => $valorTotalItem,
                        "VRTOTALITDESC" => $vrtotitemcomdesc,
                        "VRNEGOCIADO"   => $valorTotalNegociado
                    );
                    
                } else {
                    
                    $vrTotItemProp = ($valorTotalItem * $prazo_medicao) / $prazo_Integral;
                    $vrTotDescProp = ($vrtotitemcomdesc * $prazo_medicao) / $prazo_Integral;
                    $vrTotNegProp  = is_null($valorTotalNegociado) ? null : ($valorTotalNegociado * $prazo_medicao) / $prazo_Integral;
                    
                    $retorno = array(
                        "VRTOTALITEM"   => $vrTotItemProp,
                        "VRTOTALITDESC" => $vrTotDescProp,
                        "VRNEGOCIADO"   => $vrTotNegProp
                    );
                }
            }
            /* Faz o cálculo do valor proporcional em uma parcela com valor integral */
            else if ($fatuIcomp == '3') {
                
                $params = array(
                    "NRSEQCONTRATO" => $nrseqcontrato,
                    "NRORG"         => $nrorg
                );
                        
                $proRata = $this->medicaoFactory->verificaPeriodoIncompProRata($params);
                
                $dtCompetMin    = $proRata[0]['DTCOMPETMIN']; 
                $dtCompetMax    = $proRata[0]['DTCOMPETMAX']; 
                $prazoIniCont   = $proRata[0]['PRAZO_INICONT'];
                $prazo_Integral = $proRata[0]['PRAZOINT'];
                
                $params = array(
                    "NRMEDICAOFATURA" => $nrmedicaofatura,
                    "NRORG"           => $nrorg
                );
        
                $periodoMed = $this->medicaoFactory->recebeperiodoMedicao($params);
                
                $dtiniperiodo = $periodoMed[0]['DTINICOMPET']; 
                $dtfinPeriodo = $periodoMed[0]['DTFINCOMPET'];
                
                if ($dtCompetMin == $dtiniperiodo){
                    
                    if ($prazoIniCont > 0){
                        
                        $vrTotItemProp = ($valorTotalItem * $prazoIniCont) / $prazo_Integral;
                        $vrTotDescProp = ($vrtotitemcomdesc * $prazoIniCont) / $prazo_Integral;
                        
                        $vrTotNegProp = null;
                        if (!is_null($valorTotalNegociado)) {
                            $vrTotNegProp = ($valorTotalNegociado * $prazoIniCont) / $prazo_Integral;
                            $vrTotNegProp = $vrTotNegProp + $valorTotalNegociado;
                        }
                        
                        $vrTotItemProp = $vrTotItemProp + $valorTotalItem;
                        $vrTotDescProp = $vrTotDescProp + $vrtotitemcomdesc;
                        
                        $retorno = array(
                            "VRTOTALITEM"   => $vrTotItemProp,
                            "VRTOTALITDESC" => $vrTotDescProp,
                            "VRNEGOCIADO"   => $vrTotNegProp
                        );
                    }
                    else {
                        $retorno = array(
                            "VRTOTALITEM"   => $valorTotalItem,
                            "VRTOTALITDESC" => $vrtotitemcomdesc,
                            "VRNEGOCIADO"   => $valorTotalNegociado
                        );
                    }
                }
                else if ($dtCompetMax == $dtfinPeriodo){
                    
                    
                    $params = array(
                        "NRMEDICAOFATURA" => $nrmedicaofatura,
                        "NRORG"           => $nrorg
                    );
            
                    $prazos = $this->medicaoFactory->validaProporcional($params);
                    
                    $prazo_Integral = $prazos[0]["PRAZOINT"];            
                    $prazo_medicao = $prazos[0]["PRAZO"]; 
                    
                    if ($prazo_Integral == $prazo_medicao){
                        
                        $retorno = array(
                            "VRTOTALITEM"   => $valorTotalItem,
                            "VRTOTALITDESC" => $vrtotitemcomdesc,
                            "VRNEGOCIADO"   => $valorTotalNegociado
                        );
                        
                    } else {
                        
                        $vrTotItemProp = ($valorTotalItem * $prazo_medicao) / $prazo_Integral;
                        $vrTotDescProp = ($vrtotitemcomdesc * $prazo_medicao) / $prazo_Integral;
                        $vrTotNegProp  = is_null($valorTotalNegociado) ? null : ($valorTotalNegociado * $prazo_medicao) / $prazo_Integral;
                        
                        $retorno = array(
                            "VRTOTALITEM"   => $vrTotItemProp,
                            "VRTOTALITDESC" => $vrTotDescProp,
                            "VRNEGOCIADO"   => $vrTotNegProp
                        );
                    }
                    
                }
                else{
                    
                    $retorno = array(
                        "VRTOTALITEM"   => $valorTotalItem,
                        "VRTOTALITDESC" => $vrtotitemcomdesc,
                        "VRNEGOCIADO"   => $valorTotalNegociado
                    );
                }
            }
            /* Faz o cálculo do valor integral */
            else {
                
                $retorno = array(
                    "VRTOTALITEM"   => $valorTotalItem,
                    "VRTOTALITDESC" => $vrtotitemcomdesc,
                    "VRNEGOCIADO"   => $valorTotalNegociado
                );
                
            } 
        }
        else {
                
            $retorno = array(
                "VRTOTALITEM"   => $valorTotalItem,
                "VRTOTALITDESC" => $vrtotitemcomdesc,
                "VRNEGOCIADO"   => $valorTotalNegociado
            );
            
        }
        
        return $retorno;
    }

    public function updateMedicao($nrmedicaofatura, $nrorg, $dadosMedicao) {
        $situacao = $this->retornaSituacaoItem($nrmedicaofatura, $nrorg);
        
        $somaItens = $this->getSomaItMedicao($nrmedicaofatura, $nrorg);
        $vritemcont = $somaItens["VRITEMCONT"];
        $valorNegociado = $somaItens["VRNEGCONTR"];
        $valorConsiderado = $somaItens["VRCONSIDERADO"];
        
        if (!is_null($valorNegociado)) {
            $valorNegociado = $valorConsiderado;
        }
        
        $this->atualizaMedicao($vritemcont, $valorNegociado, $situacao, $nrmedicaofatura, $nrorg);
    }
    
    public function retornaSituacaoItem($nrmedicaofatura, $nrorg) {
        $situacao = $this->getSituacaoItMedicao($nrmedicaofatura, $nrorg);
         
        $pendente = $situacao["PENDENTE"];
        $parcial = $situacao["PARCIAL"];
        $calculado = $situacao["CALCULADO"];
         
        if ($pendente == 0 && $parcial == 0 && $calculado > 0) {
            $sitMed = '3';
        }
        else if ($pendente > 0 && $parcial == 0 && $calculado == 0) {
            $sitMed = '1';
        } 
        else if ($pendente = 0 && $parcial == 0 && $calculado == 0) {
            $sitMed = '1';
        } 
        else {
            $sitMed = '2';
        }
        
        return $sitMed;
    }
     
    public function getMedidasVariaveisMedicao($nrmedicaofatura, $nrorg) {
        $params = array(
            "NRMEDICAOFATURA" => $nrmedicaofatura,
            "NRORG"           => $nrorg
        );
        
        return $this->medicaoFactory->getMedidasVariaveisMedicao($params);
    }
    
    public function getItensMedicao($nrmedicaofatura, $nrorg) {
        $params = array(
            "NRMEDICAOFATURA" => $nrmedicaofatura,
            "NRORG"           => $nrorg
        );
        
        return $this->medicaoFactory->getItensMedicao($params);
    }
    
    public function getPropostaDoContrato($nrseqcontrato, $nrorg) {
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg
        );
        
        $dataset = $this->contratoFactory->getPropostaDoContrato($params);
        return (count($dataset) > 0) ? $dataset[0] : null;
    }
    
    public function getItensContrato($nrseqcontrato, $nrorg) {
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg
        );
        
        return $this->medicaoFactory->getItensContrato($params);
    }
    
    public function getSituacaoMedicaoVar($nrmedicaofatura, $cdservorca, $nrorg) {
        $params = array(
            "NRMEDICAOFATURA"   => $nrmedicaofatura,
            "CDSERVORCA"        => $cdservorca,
            "NRORG"             => $nrorg
        );
        
        $result = $this->medicaoFactory->getSituacaoMedicaoVar($params);
        
        return (count($result) > 0) ? $result[0]["PENDENTE"] : null;
    }
    
    public function updateItensMedicao($valorItem, $vritemcont, $vrnegcontr, $nrparcelasnegcontr, $situacao, $nritmedicaofatura, $nrmedicaofatura, $nritemocontrato, $cdservorca, $nrorg, $cdoperador) {
        if(is_null($vritemcont))
            $vritemcont = $valorItem;
            
        $params = array(
            "VRTOTITEMCONT"      => $valorItem,
            "VRITEMCONT"         => $vritemcont,
            "VRNEGCONTR"         => $vrnegcontr,
            "NRPARCELASNEGCONTR" => $nrparcelasnegcontr,
            "IDSITUACAO"         => $situacao,
            "CDOPERULTATU"       => $cdoperador,
            "NRITMEDICAOFATURA"  => $nritmedicaofatura,
            "NRMEDICAOFATURA"    => $nrmedicaofatura,
            "NRITEMOCONTRATO"    => $nritemocontrato,
            "CDSERVORCA"         => $cdservorca,
            "NRORG"              => $nrorg
        );
        
        return $this->medicaoFactory->updateItensMedicao($params);
    }
    
    public function getSituacaoItMedicao($nrmedicaofatura, $nrorg) {
        $params = array(
            "NRMEDICAOFATURA"   => $nrmedicaofatura,
            "NRORG"             => $nrorg
        );
        
        $result = $this->medicaoFactory->getSituacaoItMedicao($params);
        
        return (count($result) > 0) ? $result[0] : null;
    }
    
    public function getSomaItMedicao($nrmedicaofatura, $nrorg) {
        $params = array(
            "NRMEDICAOFATURA"   => $nrmedicaofatura,
            "NRORG"             => $nrorg
        );
        
        $result = $this->medicaoFactory->getSomaItMedicao($params);
        
        return (count($result) > 0) ? $result[0] : null;
    }
    
    
     public function atualizaMedicao($vritens, $vrnegcontr, $situacao, $nrmedicaofatura, $nrorg) {
        $params = array(
            "VRMEDICAO"       => $vritens,
            "VRNEGCONTR"      => $vrnegcontr,
            "IDSITUACAO"      => $situacao,
            "NRMEDICAOFATURA" => $nrmedicaofatura,
            "NRORG"           => $nrorg
        );
        
        return $this->medicaoFactory->atualizaMedicao($params);
    }
    
     /**
     * Executa a procedure "FCT_GERAR_PARCELAS_APURACAO".
     */
    public function gerarApuracaoFatura($nrmedicaofatura, $nrseqcontrato) {
        
        $nrorg = $this->environment->getNrorg();
        $operador = $this->environment->getCdOperador();
        
        $procedure = new FctGerarParcelasApuracao($this->connection);
        $procedure->executar($nrseqcontrato, $nrmedicaofatura, $nrorg, $operador);
    }
    
    /**
     * Executa a procedure "FCT_CANCELAR_MEDICAO".
     */
    public function cancelarMedicao($nrmedicaofatura, $nrseqcontrato) {
        
        $nrorg = $this->environment->getNrorg();
        $operador = $this->environment->getCdOperador();
        
        $procedure = new FctCancelarMedicao($this->connection);
        $procedure->executar($nrmedicaofatura, $nrseqcontrato, $nrorg, $operador);
    }
    
    /**
     * A função "validaFatura" valida se ja foi gerada parcela.
     */
    public function validaFatura($nrmedicaofatura, $nrseqcontrato) {
        $nrorg        = $this->environment->getNrorg();
        $params = array(
            "NRMEDICAOFATURA"  => $nrmedicaofatura,
            "NRSEQCONTRATO"    => $nrseqcontrato,
            "NRORG"            => $nrorg
        );
        
        $agrupamento = $this->medicaoFactory->getAgrupamentoCont($params);
        if($agrupamento == null)
            $dataset = $this->medicaoFactory->validaFatura($params);
        else 
            return $agrupamento;
        
        return (count($dataset) > 0) ? $dataset[0]["VALIDA"] : 0;
    }
    
    
    /**
    * Executa a procedure "FCT_CANCELAR_APURA_MEDICAO".
    */
    public function cancelarApuracaoMedicao($nrmedicaofatura, $nrseqcontrato) {
        
        $nrorg = $this->environment->getNrorg();
        $operador = $this->environment->getCdOperador();
        
        $procedure = new FctCancelarApuraMedicao($this->connection);
        $procedure->executar($nrmedicaofatura, $nrseqcontrato, $nrorg, $operador);
    }
    
    /**
    * Executa a procedure atualiza a medida variável de acordo com os dados do OM.
    */
    public function atualizarMedVariavelOM($parcelas) {
        /* 1 - Pega os dados da parcela de medição */ 
        $nrseqcontrato    = $parcelas["NRSEQCONTRATO"];
        //$nrmedicaofatura  = $parcelas["NRMEDICAOFATURA"];
        $cdservorca       = $parcelas["CDSERVORCA"];
        $dtinicompet      = $parcelas["DTINICOMPET"];
        $dtfincompet      = $parcelas["DTFINCOMPET"];
        $dtcoletaOM       = $parcelas["DTCOLETA"];
        $nrorg            = $parcelas["NRORG"];
          
        /* 2 - Cria uma nova conexão para a base da organização correta.*/ 
        $entityManagerConnection = $this->connectioncustom->updateVpdConnection($nrorg);
        
        if ($entityManagerConnection !== false) {
            $this->entityManager = $entityManagerConnection;
        }
              
        $this->connection = $this->entityManager->getConnection();
          
        $this->medicaoFactory->atualizaConnection($this->connection);
        $this->contratoFactory->atualizaConnection($this->connection);
        //$this->propostaService->atualizaConnection($this->connection);
          
        $this->connection->beginTransaction();
         
        $nrmedicaofatura = $this->getNrMedicaoFatura($nrseqcontrato, $dtinicompet, $dtfincompet, $nrorg);
    
        try {
            /* 3 - Testa se a medição já está calculada*/
            $params = array(
                "NRSEQCONTRATO" => $nrseqcontrato,
                "DTINICOMPET"   => $dtinicompet,
                "DTFINCOMPET"   => $dtfincompet,
                "NRORG"         => $nrorg,
            );
              
            $valida = $this->medicaoFactory->ValidaSitMedicao($params);
             
            $situacao = (count($valida) > 0) ? $valida[0]["IDSITUACAO"] : '0';
              
              
            /* 4 - Valida a medição retornada pelo OM, sendo:
                     Se a variável "$situacao" retornar 0, não foi encontrada nenhuma medição com os dados enviados. O sistema retorna uma mensagem de bloqueio.
                     Se a variável "$situacao" retornar 4, significa que já foi gerada uma parcela através desta medição e o cálculo não pode mais ser realizado. O sistema retorna uma mensagem de bloqueio.
                     Se a variável "$situacao" retornar 5, significa que a medição está paralisada. O sistema retorna uma mensagem de bloqueio.
                     Se a variável "$situacao" retornar 6, significa que a medição está cancelada. O sistema retorna uma mensagem de bloqueio.
                     Caso contrario a estes retornos, a rotina segue normalmente. */
              
            if (in_array($situacao, array('1', '2', '3')) && ($nrmedicaofatura != '0')) {
                $realizado = false;
                /* 5 - Pega os dados da mndida Variável */
                foreach ($parcelas["MEDVARIAVEIS"] as $parcela) {
                      
                    $cdtpmedidavariavel   = $parcela["CDTPMEDIDAVARIAVEL"];
                    $qtmedidavariavel     = $parcela["QTMEDIDAVARIAVEL"];
                    $idProdutoOM          = $parcela["NRPRODUTO"];
                    
                    $qtmedidavariavel = Utilities::stringToFloat($qtmedidavariavel);
                      
                    /* 6 - Atualiza medidas Variáveis  
                             Exceto se a situação for "3 - Calculado" e o valor já calculado for maior do que o enviado pelo OM.
                             Neste caso, é retornada uma mensagem de bloqueio da rotina.*/
                      
                    if ($situacao == '3') {
                          
                        $params = array(
                            "NRMEDICAOFATURA"    => $nrmedicaofatura,
                            "CDTPMEDIDAVARIAVEL" => $cdtpmedidavariavel, 
                            "NRORG"              => $nrorg,
                            "IDPRODUTOOM"        => $idProdutoOM,
                            "CDSERVORCA"         => $cdservorca
                        );
                          
                        $validacao = $this->medicaoFactory->ValidaQTtMedicaoConc($params);
                        if (count($validacao) == 0) {
                            throw new \Exception("Operação bloqueada! <br/>Não foi encontrado um produto na medição para atualizar a quantidade medida. Filtro utilizado: NRMEDICAOFATURA = " . $nrmedicaofatura . "; CDTPMEDIDAVARIAVEL = " . $cdtpmedidavariavel . "; NRORG = " . $nrorg . "; IDPRODUTOOM = " . $idProdutoOM . "; CDSERVORCA = " . $cdservorca);
                        }
                          
                        $quantidade = (count($validacao) > 0) ? $validacao[0]["QTMEDIDAVARIAVEL"] : 0;
                        $qtmedidavendida = (count($validacao) > 0) ? $validacao[0]["QTMEDIDAVENDIDA"] : 0;
                          
                        if ($qtmedidavariavel > $quantidade){
                              
                            if ($qtmedidavendida > $qtmedidavariavel) {
                                $qtfaturamento = $qtmedidavendida;
                            }
                            else {
                                $qtfaturamento = $qtmedidavariavel;
                            }
                                
                            $realizado = true;
                            $this->atualizaMedidaVariavelOM($qtmedidavariavel, $qtfaturamento, $nrorg, $nrmedicaofatura, $cdtpmedidavariavel, $dtcoletaOM, $idProdutoOM, $cdservorca);
                        }
                        else {
                            $realizado = false;
                            $this->atualizaUltimaColetaOM($qtmedidavariavel, $nrorg, $nrmedicaofatura, $cdtpmedidavariavel, $dtcoletaOM, $idProdutoOM, $cdservorca);
                              
                            throw new \Exception("Operação bloqueada! <br/> Valor calculado para a medição de parcela selecionada e medida variável selecionada é igual ou superior ao que está sendo enviado. <br><br> Valor Calculado: ".$quantidade." <br> Valor Enviado: ".$qtmedidavariavel);
                        }
                    }
                    else {
                        $params = array(
                            "NRMEDICAOFATURA"    => $nrmedicaofatura,
                            "CDTPMEDIDAVARIAVEL" => $cdtpmedidavariavel, 
                            "NRORG"              => $nrorg,
                            "IDPRODUTOOM"        => $idProdutoOM,
                            "CDSERVORCA"         => $cdservorca
                        );
                          
                        $validacao = $this->medicaoFactory->ValidaQTtMedicao($params);
                        if (count($validacao) == 0) {
                            throw new \Exception("Operação bloqueada! <br/>Não foi encontrado um produto na medição para atualizar a quantidade medida. Filtro utilizado: NRMEDICAOFATURA = " . $nrmedicaofatura . "; CDTPMEDIDAVARIAVEL = " . $cdtpmedidavariavel . "; NRORG = " . $nrorg . "; IDPRODUTOOM = " . $idProdutoOM . "; CDSERVORCA = " . $cdservorca);
                        }
                        
                        $qtmedidavendida = (count($validacao) > 0) ? $validacao[0]["QTMEDIDAVENDIDA"] : 0;
                          
                        if ($qtmedidavendida > $qtmedidavariavel) {
                            $qtfaturamento = $qtmedidavendida;
                        }
                        else {
                            $qtfaturamento = $qtmedidavariavel;
                        }
                          
                        $realizado = true;
                        $this->atualizaMedidaVariavelOM($qtmedidavariavel, $qtfaturamento, $nrorg, $nrmedicaofatura, $cdtpmedidavariavel, $dtcoletaOM, $idProdutoOM, $cdservorca);
                    }  
                }
                  
                /* 7 - Realiza o cálculo das medições */
                if ($realizado == true) {
                    /* Verifica se existe algum elemento na medição que ainda não foi medido.
                    Se não existir, o cálculo da medição é feito normalmente.
                    Se existir, o cálculo não é realizado, ou seja, a medição continua como pendente. */
                    $qtdElementos = $this->getQuantidadeElementosNaoMedidos($nrmedicaofatura, $nrorg);
                	if ($qtdElementos == 0) {
                	    $this->recalcularItensMedicao($nrmedicaofatura, $nrseqcontrato, $nrorg);
                	}
                }
            }
            else if (($situacao == '0') or ($nrmedicaofatura == '0')) {
                throw new \Exception("Operação bloqueada! <br/> Não foi encontrada uma medição de parcela para o contrato e período enviados. Favor entrar em contato com o administrador do sistema.");
            }
            else if ($situacao == '4') {
                throw new \Exception("Operação bloqueada! <br/> Já foram geradas as parcelas referentes a medição selecionada.");
            }
            else if ($situacao == '5') {
                throw new \Exception("Operação bloqueada! <br/> A medição selecionada está com a situação \"Paralisada\".");
            }
            else { //$situacao == '6'
                throw new \Exception("Operação bloqueada! <br/> A medição selecionada está com a situação \"Cancelada\".");
            }
            
            $this->connection->commit();
        }
        catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    public function getQuantidadeElementosNaoMedidos($nrmedicaofatura, $nrorg) {
        $params = array(
            "NRMEDICAOFATURA" => $nrmedicaofatura,
            "NRORG"           => $nrorg
        );
        
        $dataset = $this->medicaoFactory->getQuantidadeElementosNaoMedidosMedicao($params);
        return (int)$dataset[0]["QTD"];
    }
    
    /* Atualiza medidas Variáveis de acordo com os dados enviados pelo OM */
    public function atualizaMedidaVariavelOM($qtmedidavariavel, $qtfaturamento, $nrorg, $nrmedicaofatura, $cdtpmedidavariavel, $dtcoletaOM, $idProdutoOM, $cdservorca) {
          
          $operador = $this->environment->getCdOperador(); 
            	
    	  $params = array(
              "QTMEDIDAVARIAVEL"   => $qtmedidavariavel,
              "QTFATURAMENTO"      => $qtfaturamento,
              "DTCOLETA"           => $dtcoletaOM,
              "NRORG"              => $nrorg,
              "CDOPERULTATU"       => $operador,
              "NRMEDICAOFATURA"    => $nrmedicaofatura,
              "CDTPMEDIDAVARIAVEL" => $cdtpmedidavariavel,
              "IDPRODUTOOM"        => $idProdutoOM,
              "CDSERVORCA"         => $cdservorca
          );
          
          $this->medicaoFactory->atualizarMedVariavelOM($params);
    }
    
    /* Atualiza data e quantidade da última de acordo com os dados enviados pelo OM */
    public function atualizaUltimaColetaOM($qtmedidavariavel, $nrorg, $nrmedicaofatura, $cdtpmedidavariavel, $dtcoletaOM, $idProdutoOM, $cdservorca) {  
        $operador = $this->environment->getCdOperador(); 
    	$params = array(
            "DTCOLETA"           => $dtcoletaOM,
            "QTMEDIDAVARIAVEL"   => $qtmedidavariavel,
            "NRORG"              => $nrorg,
            "CDOPERULTATU"       => $operador,
            "NRMEDICAOFATURA"    => $nrmedicaofatura,
            "CDTPMEDIDAVARIAVEL" => $cdtpmedidavariavel,
            "IDPRODUTOOM"        => $idProdutoOM,
            "CDSERVORCA"         => $cdservorca
        );
          
        $this->medicaoFactory->atualizaUltimaColetaOM($params);
    }
    
    /* Busca o nr da medição de faturamento de acordo com o contrato e o período.*/
    public function getNrMedicaoFatura($nrseqcontrato, $dtinicompet, $dtfincompet, $nrorg) {  
        
    	$params = array(
              "NRSEQCONTRATO"      => $nrseqcontrato,
              "DTINICOMPET"        => $dtinicompet,
              "DTFINCOMPET"        => $dtfincompet,
              "NRORG"              => $nrorg
        );
        
        $medicao = $this->medicaoFactory->getNrMedicaoFatura($params);
        
        $retorno = (count($medicao) > 0) ? $medicao[0]["NRMEDICAOFATURA"] : 0;
        
        return $retorno;
    }
    
    public function cancelarMedicoes(array $medicoes, $dsjustificacanc, $nrorg, $cdoperador) {
        $this->connection->beginTransaction();
        try {
            foreach ($medicoes as $nrmedicaofatura) {
                $params = array(
                    "DSJUSTIFICACANC" => $dsjustificacanc,
                    "NRMEDICAOFATURA" => $nrmedicaofatura,
                    "NRORGULTATU"     => $nrorg,
                    "CDOPERULTATU"    => $cdoperador,
                    "NRORG"           => $nrorg
                );
                
                $this->medicaoFactory->atualizarMedicaoSituacaoCancelada($params);
            }
            
            $this->connection->commit();
        }
        catch (\Exception $e) {
            $this->connection->rollBack();
            throw $e;
        }
    }
    
    public function getDadosMedicao($nrmedicaofatura, $nrorg) {
        $params = array(
            "NRMEDICAOFATURA" => $nrmedicaofatura,
            "NRORG"           => $nrorg
        );
        
        $dataset = $this->medicaoFactory->getDadosMedicao($params);
        return $dataset[0];
    }
    
    public function getNegociacaoContratoParaElemento($nrseqcontrato, $dtinicompet, $dtfincompet, $nrparcnegocio, $nritemelemento, $nrorg) {
        $params = array(
            "DTINICOMPET"    => $dtinicompet,
            "DTFINCOMPET"    => $dtfincompet,
            "NRPARCNEGOCIO"  => $nrparcnegocio,
            "NRSEQCONTRATO"  => $nrseqcontrato,
            "NRITEMELEMENTO" => $nritemelemento,
            "NRORG"          => $nrorg
        );
        
        $dataset = $this->medicaoFactory->getNegociacaoContratoParaElemento($params);
        return (count($dataset) > 0) ? $dataset[0] : null;
    }
    
    public function atualizarMedidaVariavel($idcalculado, $vritemcont, $vritemcontcomdesc, $vrnegcontr, $nrparcelasnegcontr, $nrmedicaofatura, $idelementoraiz, $cdservorca, $cdtpmedidavariavel, $nrorg, $cdoperador) {
        $params = array(
            "IDCALCULADO"        => $idcalculado,
            "VRITEMCONT"         => $vritemcont,
            "VRITEMCONTCOMDESC"  => $vritemcontcomdesc,
            "VRNEGCONTR"         => $vrnegcontr,
            "NRPARCELASNEGCONTR" => $nrparcelasnegcontr,
            "NRORGULTATU"        => $nrorg,
            "CDOPERULTATU"       => $cdoperador,
            "NRMEDICAOFATURA"    => $nrmedicaofatura,
            "IDELEMENTORAIZ"     => $idelementoraiz,
            "CDSERVORCA"         => $cdservorca,
            "CDTPMEDIDAVARIAVEL" => $cdtpmedidavariavel,
            "NRORG"              => $nrorg
        );
        
        $this->medicaoFactory->atualizarMedidaVariavel($params);
    }
    
    public function getNegociacaoContratoParaServico($nrseqcontrato, $dtinicompet, $dtfincompet, $nrparcnegocio, $nritemocontrato, $nrorg) {
        $params = array(
            "DTINICOMPET"     => $dtinicompet,
            "DTFINCOMPET"     => $dtfincompet,
            "NRPARCNEGOCIO"   => $nrparcnegocio,
            "NRSEQCONTRATO"   => $nrseqcontrato,
            "NRITEMOCONTRATO" => $nritemocontrato,
            "NRORG"           => $nrorg
        );
        
        $dataset = $this->medicaoFactory->getNegociacaoContratoParaServico($params);
        return (count($dataset) > 0) ? $dataset[0] : null;
    }
    
    private function calculaValorNegociado($valorItemOriginal, $idmultipvariacao, $qtdMedida, $dadosNegociacao) {
        $valorParcelaNeg     = $dadosNegociacao["VRPARCELANEGOCIA"];
        $multiplicaQtdMedida = $dadosNegociacao["IDMULTIPLICAQTMED"];
        $tipoNegociacao      = $dadosNegociacao["IDTIPONEGOCIA"];
        
        /* IDTIPONEGOCIA
        VF - Valor Fixo
        DP - Desconto de Percentual
        DV - Desconto de Valor
        AP - Acrescimo de Percentual
        AV - Acrescimo de Valor
        */
        
        $valorNegociado = null;
        
        if ($tipoNegociacao == "VF") {
            $valorNegociado = $valorParcelaNeg;
            if ($multiplicaQtdMedida == "S" && $idmultipvariacao === "Sim") {
                $valorNegociado *= $qtdMedida;
            }
        }
        else if ($tipoNegociacao == "DP") {
            $valorNegociado = $valorItemOriginal - ($valorItemOriginal * $valorParcelaNeg / 100);
        }
        else if ($tipoNegociacao == "DV") {
            if ($multiplicaQtdMedida == "S" && $idmultipvariacao === "Sim") {
                $valorNegociado = $valorItemOriginal - ($valorParcelaNeg * $qtdMedida);
            }
            else {
                $valorNegociado = $valorItemOriginal - $valorParcelaNeg;
            }
        }
        else if ($tipoNegociacao == "AP") {
            $valorNegociado = $valorItemOriginal + ($valorItemOriginal * $valorParcelaNeg / 100);
        }
        else if ($tipoNegociacao == "AV") {
            if ($multiplicaQtdMedida == "S" && $idmultipvariacao === "Sim") {
                $valorNegociado = $valorItemOriginal + ($valorParcelaNeg * $qtdMedida);
            }
            else {
                $valorNegociado = $valorItemOriginal + $valorParcelaNeg;
            }
        }
        
        $valorNegociado = round($valorNegociado, 2);
        return $valorNegociado;
    }
    
    public function getDadosNegociacaoMedicao($nrmedicaofatura, $dtinicompet, $nrorg) {
        $params = array(
            "DTINICOMPET"     => $dtinicompet,
            "NRMEDICAOFATURA" => $nrmedicaofatura,
            "NRORG"           => $nrorg
        );
        
        $dataset = $this->medicaoFactory->getDadosNegociacaoMedicao($params);
        return $dataset[0];
    }
}

?>