/* global Util, Configuration, ZhuAclMessages */
function ZhuContainerElementsController(templateManager, ZhuAclController, ZhuAclConfig, aclEngine, ZhuAclCRUDController, MessageService){
	var self = this;
	var idErrorMessages = "";

	var getContainerElementsTreeWidget = function(){
		return templateManager.container.getWidget(ZhuAclConfig.containerElementsTreeWidgetName);
	};

	var getContainerElementsListWidget = function(){
		return templateManager.container.getWidget(ZhuAclConfig.containerElementsListWidgetName);
	};

	var clearIdErrorsMessage = function(){
		idErrorMessages = "";
	};

	var showIdErrorsMessage = function(){
		if(idErrorMessages){
			var messageConfig = {
				type: 'alert',
				icon: 'exclamation'
			};
			MessageService.showMessageByCode(ZhuAclMessages.NSHOW_WITHOUT_ID, {elementsIds: idErrorMessages}, messageConfig);
		}
	};

	var getPlannedContainer = function(rules, item, itemType, parentId){
		var plannedItems = [];
		itemType = itemType || ZhuAclConfig.containerType;
		if(item && !item.isZhuAclElement){
			var uniqueId = item.id;
			if(uniqueId){
				var aclType = ZhuAclConfig.getAclMap(itemType);
				var itemToInsert = ZhuAclController.getPlannedItemToInsertInDataSet(rules, item, aclType.aclTypeCod, parentId);
				plannedItems.push(itemToInsert);
				aclType.childs.forEach(function(type){
					if(Util.isArray(item[type])){
						item[type].forEach(function(child){
							plannedItems = plannedItems.concat(getPlannedContainer(rules, child, type, uniqueId));
						});
					}
				});
			} else {
				var nameToShow = item.label || item.name;
				idErrorMessages += "<br> " + nameToShow + ";";
			}
		}
		return plannedItems;
	};

	this.reloadContainerElementsModal = function(){
		getContainerElementsListWidget().reload();
	};

	this.reloadContainerElementsTree = function(){
		getContainerElementsTreeWidget().reload();
	};

	this.selectAccessOnChange = function(selectedRow){
		if(selectedRow && selectedRow.SELECT_ACCESS){
			ZhuAclController.setElementsListFilter(templateManager.container, selectedRow.ID, ZhuAclConfig.containerElementsTreeWidgetName);
		}
		self.reloadContainerElementsTree();
	};

	var copyItem = function(itemToCopy, itemType, parentId){
		var item;
		if(itemToCopy && !itemToCopy.isZhuAclElement){
			var uniqueId = itemToCopy.id;
			item = {
				id: uniqueId,
				name: itemToCopy.name,
				label: itemToCopy.label,
			};
			itemType = itemType || ZhuAclConfig.containerType;
			var aclType = ZhuAclConfig.getAclMap(itemType);
			aclType.childs.forEach(function(type){
				if(Util.isArray(itemToCopy[type])){
					item[type] = itemToCopy[type].map(function(child){
						return copyItem(child, type, uniqueId);
					}).filter(function(copiedChild){
						return copiedChild;
					});
				}
			});
		}
		return item;
	};

	this.containerElementsTreeAfterLoadDataSource = function(){
		clearIdErrorsMessage();
		var widget = getContainerElementsTreeWidget();
		var elementsListWidget = getContainerElementsListWidget();
		var elementsListRow = elementsListWidget && elementsListWidget.currentRow;
		var rulesData = Util.clone(widget.dataSource.data);
		widget.dataSource.data = [];

		if(elementsListRow && elementsListRow.SELECT_ACCESS){
			var rules = ZhuAclController.getRulesFromDatasource(rulesData);
			var container = copyItem(templateManager.container);
			if(container){
				aclEngine.processAcl(container, rules, templateManager.userId, templateManager.project.elementListDefaultRule || ZhuAclController.getDefaultRule());
				widget.dataSource.data = getPlannedContainer(rules, container);
				showIdErrorsMessage();
			}
		}
	};

	this.isEditableOnChange = function(currentRow){
		var widget = getContainerElementsTreeWidget();
		ZhuAclCRUDController.changeIsEditable(widget, currentRow);
	};

	this.isVisibleOnChange = function(currentRow){
		var widget = getContainerElementsTreeWidget();
		ZhuAclCRUDController.changeIsVisible(widget, currentRow);
	};
}

Configuration(function(ContextRegister){
	ContextRegister.register('ZhuContainerElementsController', ZhuContainerElementsController);
});